#!/bin/bash
# Generated by runner ({{ runnerVersion }}) at {{ timestamp }}
set -e

{% set instance_data = jsonMap(container.instanceData()) %}
{% set cluster_data = jsonMap(container.clusterData()) %}
{% set esMajorVersion = version.major() %}
{% set esMinorVersion = version.minor() %}
{% set esPatchVersion = version.patch() %}

INSTALL='/elasticsearch/bin/plugin install'

WHOAMI=$(whoami)
if [ $WHOAMI == "root" ]; then
    exec setuser founduser $0 $*
fi

export CONF_DIR="/app/config"
export LOG_DIR="/app/logs"
export DATA_DIR="/app/data"
export USER_BUNDLES_DIR="/app/found_user_bundles"
export USER_BUNDLES_SCRIPTS_DIR="$USER_BUNDLES_DIR/scripts"
export USER_BUNDLES_DICTS_DIR="$USER_BUNDLES_DIR/dictionaries"

cd /app

mkdir -p logs
BOOT_LOG_FILE=logs/boot.log
# save stdout and stderr for later
exec 3>&1 4>&2
exec >  >(ts '%Y-%m-%dT%H:%M:%S%z' |tee -a $BOOT_LOG_FILE)
exec 2> >(ts '%Y-%m-%dT%H:%M:%S%z' |tee -a $BOOT_LOG_FILE >&2)

echo "Booting at $(date)"

mkdir -p plugins

# Without this setting, glibc will base the number of malloc arenas on the number of cores, which can hurt small processes running on large servers.
# Heroku has done a decent job benchmarking the effects of this setting, weighing performance of mallocs and actual memory used.
# A setting of 4 seems like a good tradeoff for our needs. https://devcenter.heroku.com/articles/testing-cedar-14-memory-use
export MALLOC_ARENA_MAX=4

# Number of GC threads should depend on the size of the node.
# Tiered compilation does not help Elasticsearch much, and reserves 48M memory for additional code caches.
# Disabling it reduces the code cache, however, which is problematic: https://github.com/elastic/cloud/issues/4603
export ES_JAVA_OPTS="-XX:ParallelGCThreads={{ container.parallelGCThreads() }} -XX:ConcGCThreads={{ container.concGCThreads() }}"

export ES_MAX_MEM={{ container.heapSize() }}M
export ES_MIN_MEM={{ container.heapSize() }}M
export ES_HEAP_SIZE={{ container.heapSize() }}M

# Use quota aware fs if included in this image.
if [ -f /elasticsearch/lib/quota-aware-fs-*.jar ]; then
    echo "Enabling QuotaAwareFileSystemProvider"
    export ES_JAVA_OPTS="$ES_JAVA_OPTS -Djava.nio.file.spi.DefaultFileSystemProvider=co.elastic.cloud.quotaawarefs.QuotaAwareFileSystemProvider"
fi

# ES docker images built before 2016-02-26 may not have libjvmkill installed.
if [ -e /usr/lib/libjvmkill.so ]; then
    export ES_JAVA_OPTS="$ES_JAVA_OPTS -agentpath:/usr/lib/libjvmkill.so"
else
    export ES_JAVA_OPTS="$ES_JAVA_OPTS -XX:OnOutOfMemoryError=/app/on-error-exitcode.sh"
fi

# Less verbose curator
export ES_JAVA_OPTS="$ES_JAVA_OPTS -Dcurator-log-only-first-connection-issue-as-error-level=true"

export PLUGIN_TARGET_DIR="/app/plugins"
export PLUGIN_SOURCE_DIR="/elasticsearch/plugins"

# In case of in-place version upgrades, make sure we clean up the plugins directory
# if this is a different version
export PLUGIN_SET_MARKER="$PLUGIN_TARGET_DIR/.found_installed_plugins-{{ pluginSetIdentifier }}"
if [ ! -e $PLUGIN_SET_MARKER ]; then
    rm -rf $PLUGIN_TARGET_DIR/*
fi

{% for plugin_name in instance_data.elasticsearch.plugins %}
    # if the plugin does not exist copy from bundle. Symlinking does not work since 2.x
    if [ ! -e $PLUGIN_TARGET_DIR/{{ plugin_name }} ]; then
        cp -r $PLUGIN_SOURCE_DIR/{{ plugin_name }} $PLUGIN_TARGET_DIR/{{ plugin_name }} || true
    fi
{%  endfor %}

echo "Installing user plugins."
{% for user_plugin in instance_data.elasticsearch.user_plugins %}
    {% set name = user_plugin.name  %}
    {% set url = user_plugin.url %}
    # installing user plugin: {{ name }}
    if [ ! -e $PLUGIN_TARGET_DIR/{{ name }} ] && [ ! -e $PLUGIN_TARGET_DIR/{{ name }}/.found_installed_plugin ]; then
        echo "Installing {{ name }}..."
        rm -rf $PLUGIN_TARGET_DIR/{{ name }} || true

            wget --no-verbose -O plugins/{{ name }}.zip '{{ url }}' \
                && unzip -o plugins/{{ name }}.zip -d $PLUGIN_TARGET_DIR/{{ name }} \
                || false
            rm plugins/{{ name }}.zip

        echo "Installed {{ name }}."
    else
        if [ -e $PLUGIN_TARGET_DIR/{{ name }}/.found_installed_plugin ]; then
            echo "Already installed {{ name }}."
        else
            echo "Not installing {{ name }} because it already exists as a built-in, enabled plugin"
        fi
    fi
{% endfor %}

# Indicate that plugins should be ok for this version, so we don't wipe everything above
touch $PLUGIN_SET_MARKER

echo "Installing user bundles."

mkdir -p $USER_BUNDLES_DIR
{% if not container.shouldInstallUserBundles() %}
echo "Skipping installation of user bundles due to allocation configuration."
{% elif not instance_data.elasticsearch.user_bundles %}
echo "No user bundles defined"
{% else %}
if [ ! -e $USER_BUNDLES_DIR/.found_installed_all_bundles ]; then
    {% for user_bundle in instance_data.elasticsearch.user_bundles %}
        {% set name = user_bundle.name %}
        if [ ! -e $USER_BUNDLES_DIR/.found_installed_bundle_{{ name }} ]; then
            echo "Installing {{ name }}..."

            wget -O {{ name }}.zip '{{ user_bundle['url'] }}' \
                && unzip -o {{ name }}.zip -d $USER_BUNDLES_DIR/ \
                || false

            echo "Installed {{ name }}."
            touch $USER_BUNDLES_DIR/.found_installed_bundle_{{ name }}
            rm -f {{ name }}.zip
        else
            echo "Already installed {{ name }}."
        fi
    {% endfor %}
    touch $USER_BUNDLES_DIR/.found_installed_all_bundles
else
    echo "Not installing found_user_bundles because they are already installed"
fi

# Copy the scripts directory to the found_user_bundles plugin
[[ -h $CONF_DIR/scripts ]] && rm $CONF_DIR/scripts
if [ -e $USER_BUNDLES_SCRIPTS_DIR ]; then
    cp -r $USER_BUNDLES_SCRIPTS_DIR $CONF_DIR/scripts
fi

# Copy the dictionaries directory from the found_user_bundles
[[ -h $CONF_DIR/dictionaries ]] && rm $CONF_DIR/dictionaries
if [ -e $USER_BUNDLES_DICTS_DIR ]; then
    # We copy the dictionaries directly to /app/config. We need `-n` in order not to override existing files
    echo "Installing dictionaries"
    cp -rn $USER_BUNDLES_DICTS_DIR/* $CONF_DIR
fi

# Maybe an old-style bundle
old_style_items=$(find $USER_BUNDLES_DIR -maxdepth 1 \
        -not -path $USER_BUNDLES_DIR \
        -not -path $USER_BUNDLES_SCRIPTS_DIR \
        -not -path $USER_BUNDLES_DICTS_DIR \
        -not -path "$USER_BUNDLES_DIR/\.found*")
for item in $old_style_items; do
    echo "Installing old-style-bundle dictionary: $item"
    cp -rn "$item" $CONF_DIR
done
{% endif %}

echo "Done installing plugins and bundles, verifying required config files."

echo "Done verifying required config files, starting Elasticsearch. See Elasticsearch logs for further output."

export CONFIG_OPTIONS="-Des.default.path.logs=${LOG_DIR} -Des.default.path.data=${DATA_DIR} -Des.default.path.conf=${CONF_DIR}"

# used by metricbeat
export ECE_COMPONENT="elasticsearch"
export ECE_CLUSTER_NAME="{{ container.id().group() }}"
export ECE_INSTANCE_ID="{{ container.id().name() }}"
export ECE_CLUSTER_VERSION="{{ container.versionString() }}"

# restore stdout and stderr
exec 1>&3 2>&4

# Do not abort script if elasticsearch returns error code
set +e
/elasticsearch/bin/elasticsearch -Dfound.cluster_name={{ container.id().group() }} -Dfound.instance_name={{ container.id().name() }} -p /app/es.pid $CONFIG_OPTIONS $*
ES_EXIT_CODE=$?
if [ "$ES_EXIT_CODE" -ge "1" ]; then
    echo "Elasticsearch exited with status $ES_EXIT_CODE, running ./on-error-exitcode.sh $ES_EXIT_CODE"
    ./on-error-exitcode.sh $ES_EXIT_CODE
fi
exit $ES_EXIT_CODE
